@@ -64,7 +64,6 @@ const char *sqlite3TestInit(Tcl_Interp *interp){
   extern int Sqlitetest4_Init(Tcl_Interp*);
   extern int Sqlitetest5_Init(Tcl_Interp*);
   extern int Sqlitetest6_Init(Tcl_Interp*);
-  extern int Sqlitetest7_Init(Tcl_Interp*);
   extern int Sqlitetest8_Init(Tcl_Interp*);
   extern int Sqlitetest9_Init(Tcl_Interp*);
   extern int Sqlitetestasync_Init(Tcl_Interp*);
@@ -136,7 +135,6 @@ const char *sqlite3TestInit(Tcl_Interp *interp){
   Sqlitetest4_Init(interp);
   Sqlitetest5_Init(interp);
   Sqlitetest6_Init(interp);
-  Sqlitetest7_Init(interp);
   Sqlitetest8_Init(interp);
   Sqlitetest9_Init(interp);
   Sqlitetestasync_Init(interp);
