@@ -9,6 +9,14 @@
 # Then add the extra "case PragTyp_XXXXX:" and subsequent code for the
 # new pragma in ../src/pragma.c.
 #
+# The results are normally written into the ../src/pragma.h file.  However,
+# if an alternative output file name is provided as an argument, then
+# results are written into the alternative.  For example:
+#
+#     tclsh tool/mkpragmatab.tcl            ;# <--- Results to src/pragma.h
+#
+#     tclsh tool/mkpragmatab.tcl /dev/tty   ;# <-- results to terminal
+#
 
 # Flag meanings:
 set flagMeaning(NeedSchema) {Force schema load before running}
@@ -231,6 +239,12 @@ set pragma_def {
   COLS: cid name type notnull dflt_value pk hidden
   IF:   !defined(SQLITE_OMIT_SCHEMA_PRAGMAS)
 
+  NAME: table_list
+  TYPE: TABLE_LIST
+  FLAG: NeedSchema Result1
+  COLS: schema name type ncol wr strict
+  IF:   !defined(SQLITE_OMIT_SCHEMA_PRAGMAS)
+
   NAME: stats
   FLAG: NeedSchema Result0 SchemaReq
   COLS: tbl idx wdth hght flgs
@@ -256,7 +270,7 @@ set pragma_def {
   IF:   !defined(SQLITE_OMIT_SCHEMA_PRAGMAS)
 
   NAME: database_list
-  FLAG: NeedSchema Result0
+  FLAG: Result0
   COLS: seq name file
   IF:   !defined(SQLITE_OMIT_SCHEMA_PRAGMAS)
 
@@ -304,12 +318,12 @@ set pragma_def {
   IF:   !defined(SQLITE_OMIT_CASE_SENSITIVE_LIKE_PRAGMA)
 
   NAME: integrity_check
-  FLAG: NeedSchema Result0 Result1
+  FLAG: NeedSchema Result0 Result1 SchemaOpt
   IF:   !defined(SQLITE_OMIT_INTEGRITY_CHECK)
 
   NAME: quick_check
   TYPE: INTEGRITY_CHECK
-  FLAG: NeedSchema Result0 Result1
+  FLAG: NeedSchema Result0 Result1 SchemaOpt
   IF:   !defined(SQLITE_OMIT_INTEGRITY_CHECK)
 
   NAME: encoding
@@ -396,8 +410,12 @@ set pragma_def {
 
 # Open the output file
 #
-set destfile "[file dir [file dir [file normal $argv0]]]/src/pragma.h"
-puts "Overwriting $destfile with new pragma table..."
+if {$argc>0} {
+  set destfile [lindex $argv 0]
+} else {
+  set destfile "[file dir [file dir [file normal $argv0]]]/src/pragma.h"
+  puts "Overwriting $destfile with new pragma table..."
+}
 set fd [open $destfile wb]
 puts $fd {/* DO NOT EDIT!
 ** This file is automatically generated by the script at
