@@ -99,6 +99,9 @@ while test "$1" != ""; do
     --stmtcache)
         shift; SPEEDTEST_OPTS="$SPEEDTEST_OPTS --stmtcache $1"
         ;;
+    --nolongdouble)
+        SPEEDTEST_OPTS="$SPEEDTEST_OPTS --nolongdouble"
+        ;;
     --checkpoint)
         SPEEDTEST_OPTS="$SPEEDTEST_OPTS --checkpoint"
         ;;
@@ -158,6 +161,9 @@ while test "$1" != ""; do
     --fp)
         SPEEDTEST_OPTS="$SPEEDTEST_OPTS --testset fp"
         ;;
+    --stmtscanstatus)
+        SPEEDTEST_OPTS="$SPEEDTEST_OPTS --stmtscanstatus"
+        ;;
     -*)
         CC_OPTS="$CC_OPTS $1"
         ;;
